<?php

namespace App\DataFixtures;

use App\Entity\Brand;
use App\Entity\Car;
use App\Entity\Choice;
use App\Entity\Question;
use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\DBAL\Logging\DebugStack;
use Doctrine\DBAL\Logging\EchoSQLLogger;
use Doctrine\ORM\EntityManager;
use Doctrine\Persistence\ObjectManager;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AppFixtures extends Fixture implements ContainerAwareInterface
{
    /** @var string */
    private $environment; // dev, test
    /** @var EntityManager */
    private $em;
    /** @var ContainerInterface */
    private $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
        $kernel = $this->container->get('kernel');
        if ($kernel) $this->environment = $kernel->getEnvironment();
    }


    public function load(ObjectManager $manager)
    {
        echo "HELLO, FIXTURES\n";
        $this->em = $manager;

        $stackLogger = new DebugStack();
        $echoLogger = new EchoSQLLogger();
        $this->em->getConnection()->getConfiguration()->setSQLLogger($stackLogger);

        $question1 = new Question();
        $question1->setQuText("Do you like PHP?");
        $this->em->persist($question1);

        $question2 = new Question();
        $question2->setQuText("What is your favourite color?");
        $this->em->persist($question2);
        $this->em->flush();
        echo "QUESTIONS OK. QUERIES: ".count($stackLogger->queries)."\n";

        $ans1 = new Choice();
        $ans1->setChoVisible(true);
        $ans1->setChoNumvotes(0);
        $ans1->setChoText("YES");
        $ans1->setChoQuestion($question1);
        $this->em->persist($ans1);

        $ans2 = new Choice();
        $ans2->setChoVisible(true);
        $ans2->setChoNumvotes(0);
        $ans2->setChoText("NO");
        $ans2->setChoQuestion($question1);
        $this->em->persist($ans2);

        $ans3 = new Choice();
        $ans3->setChoVisible(true);
        $ans3->setChoNumvotes(0);
        $ans3->setChoText("RED");
        $ans3->setChoQuestion($question2);
        $this->em->persist($ans3);

        $ans4 = new Choice();
        $ans4->setChoVisible(true);
        $ans4->setChoNumvotes(0);
        $ans4->setChoText("WHITE");
        $ans4->setChoQuestion($question2);
        $this->em->persist($ans4);

        $ans5 = new Choice();
        $ans5->setChoVisible(true);
        $ans5->setChoNumvotes(0);
        $ans5->setChoText("GREEN");
        $ans5->setChoQuestion($question2);
        $this->em->persist($ans5);

        $this->em->flush();
        echo "CHOICES OK. QUERIES: ".count($stackLogger->queries)."\n";
        echo "\n\n";

        $oneChoice = $this->em->getRepository(Choice::class)->findOneBy(["cho_text"=>"NO"]);
        $oneChoiceId = $oneChoice->getChoId();
        echo "CHOICE #{$oneChoiceId} FETCHED\n";
        $oneChoice->setChoNumvotes(42); // PROXY CLASS
        $this->em->persist($oneChoice);
        $this->em->flush();
        // $numVotes=0;
        $numVotes = $this->em->getRepository(Choice::class)->find($oneChoiceId)->getChoNumvotes();
        echo "MOD OK. VOTES: {$numVotes}. QUERIES: ".count($stackLogger->queries)."\n";
        echo "\n\n";

        // IDENTITY MAP
        echo "NUMBER OF CHOICES FOR QUESTION#1\n";
        $questionId = $question1->getQuId();
        echo $question1->getQuChoices()->count()." BEFORE1\n";
        echo $this->em->getRepository(Question::class)->find($questionId)->getQuChoices()->count()." BEFORE2\n";
        $this->em->clear();
        echo $this->em->getRepository(Question::class)->find($questionId)->getQuChoices()->count()." AFTER\n";
        echo "\n\n";

        $oneChoice = $this->em->getRepository(Choice::class)->find($oneChoiceId);
        $this->em->remove($oneChoice);
        $this->em->flush();
        echo "DEL OK. QUERIES: ".count($stackLogger->queries)."\n";
        echo "\n\n";

        $bmw = new Brand();
        $bmw->setBrandName("BMW");
        $this->em->persist($bmw);

        $audi = new  Brand();
        $audi->setBrandName("Audi");
        $this->em->persist($audi);

        $citroen = new Brand();
        $citroen->setBrandName("Citroen");
        $this->em->persist($citroen);
        $this->em->flush();
        echo "BRANDS OK. QUERIES: ".count($stackLogger->queries)."\n";

        $car = new Car();
        $car->setCarBrand($bmw);
        $car->setCarModel("i8");
        $car->setCarPrice(120000);
        $car->setCarVisible(true);
        $this->em->persist($car);
        $this->em->flush();
        echo "CAR OK. QUERIES: ".count($stackLogger->queries)."\n";
    }
}
// CRUD
// Create, Read, Update, Delete

/*
php bin/console doctrine:database:create

php bin/console doctrine:schema:drop --force --full-database
php bin/console doctrine:schema:update --dump-sql
php bin/console doctrine:schema:update --force

php bin/console doctrine:fixtures:load --no-interaction -vvv
 */
