<?php
// php bin/console doctrine:fixtures:load --no-interaction -vvv
// composer require --dev symfony/phpunit-bridge
// php bin/phpunit

namespace App\Tests\Service;

use App\Entity\Brand;
use App\Entity\Car;
use App\Service\CarService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;

class CarServiceTest extends WebTestCase
{
    public function testIsTrue(){
        // AAA = Arrange + Act + Assert
        $this->assertEquals(42, 42);
        self::assertEquals(15, 15);
    }

    /** @var EntityManagerInterface */
    private static $em;
    /** @var CarService */
    private static $carService;
    /** @var Brand */
    private $bmw;

    public static function setUpBeforeClass()
    {
        exec("php bin/console doctrine:fixtures:load --no-interaction");
        self::bootKernel();
        self::$em = self::$kernel->getContainer()->get('doctrine')->getManager();
        self::$carService = self::$kernel->getContainer()->get('test.carservice'); // hidden IoC?
        self::ensureKernelShutdown();
    }

    public function setUp()
    {
        $this->bmw = self::$em->getRepository(Brand::class)->findOneBy(['brand_name'=>'BMW']);
    }

    public function testBmwMustExist(){
        $this->assertNotNull($this->bmw);
    }

    public function testCanFetchAllCars(){
        $cars = self::$carService->getAllCars();
        $this->assertEquals(1, count($cars));
    }

    private function createCar() : Car
    {
        $car = new Car();
        $car->setCarBrand($this->bmw);
        $car->setCarModel("M5");
        $car->setCarPrice(123456);
        $car->setCarVisible(true);
        return $car;
    }

    public function testCanAddCar() : Car
    {
        $oneCar = $this->createCar();

        self::$carService->saveCar($oneCar);
        $allCars = self::$carService->getAllCars();
        $this->assertEquals(2, count($allCars));

        $carFromDb = self::$em->getRepository(Car::class)->findOneBy(["car_model"=>"M5"]);
        $this->assertNotNull($carFromDb);
        return $oneCar;
    }

    /**
     * @param Car $oneCar
     * @depends testCanAddCar
     */
    public function testCanRemoveAfterAdd(Car $oneCar)
    {
        self::$carService->removeCar($oneCar->getCarId());

        self::$em->clear();
        $allCars = self::$carService->getAllCars();
        $this->assertEquals(1, count($allCars));

        $carFromDb = self::$em->getRepository(Car::class)->findOneBy(["car_model"=>"M5"]);
        $this->assertNull($carFromDb);
    }

    public function testApiEndpoint() // should be Test/Controller/CarControllerTest
    {
        $client = self::createClient();
        $crawler = $client->request("GET", "/cars/api/list");
        // $this->assertContains("SomeContent", $crawler->filter("#testDiv p")->text());
        $response = $client->getResponse();

        $this->assertEquals(200, $response->getStatusCode());

        $array = json_decode($response->getContent(), true);
        $this->assertNotEmpty($array);
        $this->assertTrue(isset($array[0]["car_model"]));
        $this->assertEquals("i8", $array[0]["car_model"]);
    }
}

// IMPORTANT: inlude the FULL SYMFONY PROJECT DIRECTORY in the git repository!!!
// VERIFY:
// git clone https://user@git.address.com/some.git
// cd someDir
// composer install
// should be able to access this website in your browser
// only after server:start
